@@ -35,6 +35,7 @@ from uuid import uuid4
 import sys
 import textwrap
 import types
+import warnings
 
 import pygame
 import pygame.gfxdraw as gfxdraw
@@ -61,8 +62,6 @@ class Menu(object):
     :type title: str
     :param center_content: Auto centers the menu on the vertical position after a widget is added/deleted
     :type center_content: bool
-    :param back_box: Draw a back-box button on header
-    :type back_box: bool
     :param column_force_fit_text: Force text fitting of widgets if the width exceeds the column max width
     :type column_force_fit_text: bool
     :param column_max_width: List/Tuple representing the max width of each column in px, None equals no limit
@@ -89,13 +88,13 @@ class Menu(object):
     :type rows: int, None
     :param theme: Menu theme object, if None use the default theme
     :type theme: :py:class:`pygame_menu.themes.Theme`
+    :param kwargs: Optional keyword parameters
     """
 
     def __init__(self,
                  height,
                  width,
                  title,
-                 back_box=True,
                  center_content=True,
                  column_force_fit_text=False,
                  column_max_width=None,
@@ -110,10 +109,10 @@ class Menu(object):
                  onclose=None,
                  rows=None,
                  theme=_themes.THEME_DEFAULT,
+                 **kwargs
                  ):
         assert isinstance(height, (int, float))
         assert isinstance(width, (int, float))
-        assert isinstance(back_box, bool)
         assert isinstance(center_content, bool)
         assert isinstance(column_force_fit_text, bool)
         assert isinstance(column_max_width, (tuple, type(None), (int, float), list))
@@ -132,6 +131,9 @@ class Menu(object):
         # Assert theme
         theme.validate()
 
+        # Assert pygame was initialized
+        assert pygame.get_init(), 'pygame is not initialized'
+
         # Column/row asserts
         assert columns >= 1, 'number of columns must be greater or equal than 1'
         if columns > 1:
@@ -160,9 +162,14 @@ class Menu(object):
             'menu width and height must be greater than zero'
 
         # Get window size
-        window_width, window_height = pygame.display.get_surface().get_size()
+        surface = pygame.display.get_surface()
+        if surface is None:
+            msg = 'pygame surface could not be retrieved, check if pygame.display.set_mode() was called'
+            raise RuntimeError(msg)
+        window_width, window_height = surface.get_size()
         assert width <= window_width and height <= window_height, \
-            'menu size must be lower than the size of the window'
+            'menu size ({0}x{1}) must be lower than the size of the window ({2}x{3})'.format(
+                width, height, window_width, window_height)
 
         # Generate ID if empty
         if len(menu_id) == 0:
@@ -242,6 +249,14 @@ class Menu(object):
         self._mouse_visible_default = mouse_visible
 
         # Create Menu bar (title)
+        back_box = kwargs.get('back_box', None)
+        if back_box is not None:  # Check compatibility
+            assert isinstance(back_box, bool)
+            msg = 'back_box Menu constructor parameter moved to Theme.menubar_close_button. ' \
+                  'back_box will be removed in v3.3'
+            warnings.warn(msg)
+        else:  # Use theme
+            back_box = theme.menubar_close_button
         self._menubar = _widgets.MenuBar(
             title=title,
             width=self._width,
@@ -289,6 +304,11 @@ class Menu(object):
             shadow_position=self._theme.scrollbar_shadow_position
         )
 
+        # Upon this, no more kwargs should exist, raise exception if there's more
+        for invalid_keyword in kwargs.keys():
+            msg = 'Menu constructor parameter {} does not exist'.format(invalid_keyword)
+            raise ValueError(msg)
+
     def get_current(self):
         """
         Get current active Menu. If the user has not opened any submenu
@@ -332,7 +352,7 @@ class Menu(object):
         :param title: Title of the button
         :type title: str
         :param action: Action of the button, can be a Menu, an event or a function
-        :type action: :py:class:`pygame_menu.Menu`, :py:class:`pygame_menu.events.MenuAction`, callable
+        :type action: :py:class:`pygame_menu.Menu`, :py:class:`pygame_menu.events.MenuAction`, callable, None
         :param args: Additional arguments used by a function
         :type args: any
         :param kwargs: Additional keyword arguments
@@ -340,7 +360,7 @@ class Menu(object):
         :return: Widget object
         :rtype: :py:class:`pygame_menu.widgets.Button`
         """
-        assert isinstance(title, str)
+        title = str(title)
 
         # Filter widget attributes to avoid passing them to the callbacks
         attributes = self._filter_widget_attributes(kwargs)
@@ -349,6 +369,9 @@ class Menu(object):
         button_id = kwargs.pop('button_id', '')
         assert isinstance(button_id, str), 'ID must be a string'
 
+        if action is None:
+            action = _events.NONE
+
         # If element is a Menu
         onchange = None
         if isinstance(action, Menu):
@@ -450,7 +473,7 @@ class Menu(object):
             onchange=onchange,
             onreturn=onreturn,
             prev_size=previsualization_width,
-            title=title,
+            title=str(title),
             **kwargs
         )
 
@@ -518,7 +541,7 @@ class Menu(object):
                   title,
                   label_id='',
                   max_char=0,
-                  selectable=False,
+                  selectable=True,
                   **kwargs):
         """
         Add a simple text to the Menu.
@@ -553,6 +576,8 @@ class Menu(object):
         assert isinstance(max_char, int)
         assert isinstance(selectable, bool)
         assert max_char >= -1
+
+        title = str(title)
         if len(label_id) == 0:
             label_id = str(uuid4())  # If wrap
 
@@ -640,6 +665,7 @@ class Menu(object):
         :return: Widget object
         :rtype: :py:class:`pygame_menu.widgets.Selector`
         """
+
         # Filter widget attributes to avoid passing them to the callbacks
         attributes = self._filter_widget_attributes(kwargs)
 
@@ -649,7 +675,7 @@ class Menu(object):
             onchange=onchange,
             onreturn=onreturn,
             selector_id=selector_id,
-            title=title,
+            title=str(title),
             **kwargs
         )
 
@@ -756,7 +782,7 @@ class Menu(object):
             password=password,
             tab_size=tab_size,
             textinput_id=textinput_id,
-            title=title,
+            title=str(title),
             valid_chars=valid_chars,
             **kwargs
         )
