@@ -278,10 +278,9 @@ class MenuBar(Widget):
         :type offsety: int, float
         :return: None
         """
-        assert isinstance(title, str)
         assert isinstance(offsetx, (int, float))
         assert isinstance(offsety, (int, float))
-        self._title = title
+        self._title = str(title)
         self._offsety = offsety
         self._offsetx = offsetx
 
