@@ -35,7 +35,7 @@ import math
 import pygame
 import pygame_menu.controls as _controls
 import pygame_menu.locals as _locals
-from pygame_menu.utils import check_key_pressed_valid, make_surface
+from pygame_menu.utils import check_key_pressed_valid, make_surface, assert_color
 from pygame_menu.widgets.core import Widget
 
 try:
@@ -52,6 +52,7 @@ except ImportError:
         """
         pass
 
+
     def paste():
         """
         Paste method.
@@ -61,6 +62,7 @@ except ImportError:
         """
         return ''
 
+
     class PyperclipException(RuntimeError):
         """
         Pyperclip exception thrown by pyperclip.
@@ -72,8 +74,8 @@ class TextInput(Widget):
     """
     Text input widget.
 
-    :param label: Input label text
-    :type label: str
+    :param title: Text input title
+    :type title: str
     :param textinput_id: ID of the text input
     :type textinput_id: str
     :param input_type: Type of data
@@ -121,13 +123,13 @@ class TextInput(Widget):
     """
 
     def __init__(self,
-                 label='',
+                 title='',
                  textinput_id='',
                  input_type=_locals.INPUT_TEXT,
                  input_underline='',
                  copy_paste_enable=True,
                  cursor_color=(0, 0, 0),
-                 cursor_selection_color=(30, 30, 30),
+                 cursor_selection_color=(30, 30, 30, 100),
                  cursor_selection_enable=True,
                  history=50,
                  maxchar=0,
@@ -145,7 +147,7 @@ class TextInput(Widget):
                  valid_chars=None,
                  *args,
                  **kwargs):
-        assert isinstance(label, str)
+        assert isinstance(title, str)
         assert isinstance(textinput_id, str)
         assert isinstance(input_type, str)
         assert isinstance(input_underline, str)
@@ -170,34 +172,46 @@ class TextInput(Widget):
         assert tab_size >= 0, 'tab size must be equal or greater than zero'
         assert len(password_char) == 1, 'password char must be a character'
 
-        super(TextInput, self).__init__(widget_id=textinput_id,
-                                        onchange=onchange,
-                                        onreturn=onreturn,
-                                        args=args,
-                                        kwargs=kwargs)
+        assert_color(cursor_color)
+        assert_color(cursor_selection_color)
+        if pygame.vernum.major == 2:
+            assert len(cursor_selection_color) == 4, 'cursor selection color alpha must be defined'
+            assert cursor_selection_color[3] != 255, 'cursor selection color alpha cannot be opaque'
+
+        super(TextInput, self).__init__(
+            title=title,
+            widget_id=textinput_id,
+            onchange=onchange,
+            onreturn=onreturn,
+            args=args,
+            kwargs=kwargs
+        )
 
         self._input_string = ''  # Inputted text
-        self._ignore_keys = (  # Ignore keys on input-gathering events
+        self._ignore_keys = (  # Ignore keys on keyrepeat event
             _controls.KEY_MOVE_DOWN,
             _controls.KEY_MOVE_UP,
             pygame.K_CAPSLOCK,
+            pygame.K_END,
+            pygame.K_ESCAPE,
+            pygame.K_HOME,
             pygame.K_LCTRL,
             pygame.K_LSHIFT,
             pygame.K_NUMLOCK,
             pygame.K_RCTRL,
             pygame.K_RETURN,
             pygame.K_RSHIFT,
-            pygame.K_TAB
+            pygame.K_TAB,
         )
 
         # Vars to make keydowns repeat after user pressed a key for some time:
+        self._absolute_origin = (0, 0)  # To calculate mouse collide point
         self._block_copy_paste = False  # Blocks event
         self._key_is_pressed = False
         self._keyrepeat_counters = {}  # {event.key: (counter_int, event.unicode)} (look for "***")
         self._keyrepeat_initial_interval_ms = repeat_keys_initial_ms
         self._keyrepeat_interval_ms = repeat_keys_interval_ms
         self._last_key = 0  # type: int
-        self._absolute_origin = (0, 0)  # To calculate mouse collide point
 
         # Mouse handling
         self._keyrepeat_mouse_ms = 0.0  # type: float
@@ -255,8 +269,6 @@ class TextInput(Widget):
         self._input_underline = input_underline
         self._input_underline_size = 0.0  # type: float
         self._keychar_size = {'': 0}  # type: dict
-        self._label = label
-        self._label_size = 0.0  # type: float
         self._last_char = ''  # type: str
         self._last_rendered_string = '__pygame_menu__last_render_string__'  # type: str
         self._last_rendered_surface = None  # type: (pygame.Surface,None)
@@ -269,20 +281,24 @@ class TextInput(Widget):
         self._password = password
         self._password_char = password_char
         self._tab_size = tab_size
+        self._title_size = 0.0  # type: float
 
     def _apply_font(self):
         self._ellipsis_size = self._font.size(self._ellipsis)[0]
-        self._label_size = self._font.size(self._label)[0]
+        self._title_size = self._font.size(self._title)[0]
 
         # Generate the underline surface
         self._input_underline_size = self._font.size(self._input_underline)[0]
 
         # Size of maxwidth if not zero
-        self._maxwidthsize = self.font_render_string('O' * self._maxwidth_base).get_size()[0]
+        max_char = 'O'
+        if self._password:
+            max_char = self._password_char
+        self._maxwidthsize = self._font_render_string(max_char * self._maxwidth_base).get_size()[0]
 
         # Update password char size
         if self._password:
-            password_size = self.font_render_string(self._password_char).get_size()[0]
+            password_size = self._font_render_string(self._password_char).get_size()[0]
             if password_size == 0:
                 raise ValueError(
                     'Password character is not valid, the size of the font is zero, use another character or change the font')
@@ -330,12 +346,14 @@ class TextInput(Widget):
         # Draw background color
         self._fill_background_color(surface)
 
-        # Draw selection first
-        if self._selection_surface is not None:
-            surface.blit(self._selection_surface, (self._selection_position[0], self._selection_position[1]))
-
-        # Draw string
-        surface.blit(self._surface, (self._rect.x, self._rect.y))
+        if pygame.vernum.major == 2:
+            surface.blit(self._surface, (self._rect.x, self._rect.y))  # Draw string
+            if self._selection_surface is not None:  # Draw selection
+                surface.blit(self._selection_surface, (self._selection_position[0], self._selection_position[1]))
+        else:
+            if self._selection_surface is not None:  # Draw selection
+                surface.blit(self._selection_surface, (self._selection_position[0], self._selection_position[1]))
+            surface.blit(self._surface, (self._rect.x, self._rect.y))  # Draw string
 
         # Draw cursor
         if self.selected and self._cursor_surface and \
@@ -344,7 +362,7 @@ class TextInput(Widget):
                                                 self._rect.y + self._cursor_surface_pos[1]))
 
     def _render(self):
-        string = self._label + self._get_input_string()  # Render string
+        string = self._title + self._get_input_string()  # Render string
 
         if not self._render_hash_changed(self._menu.get_id(), string, self.selected, self._cursor_render,
                                          self._selection_enabled, self.active):
@@ -403,8 +421,8 @@ class TextInput(Widget):
             string_init = string[self._renderbox[0]:pos[0]]
             string_final = string[self._renderbox[0]:pos[1]]
 
-            x1 = self._cursor_offset + self._font.size(self._label + string_init)[0]
-            x2 = self._cursor_offset + self._font.size(self._label + string_final)[0] + 1
+            x1 = self._cursor_offset + self._font.size(self._title + string_init)[0]
+            x2 = self._cursor_offset + self._font.size(self._title + string_final)[0] + 1
 
             self._last_selection_render[0] = self._selection_box[0]
             self._last_selection_render[1] = self._selection_box[1]
@@ -413,7 +431,7 @@ class TextInput(Widget):
             if x <= 1:
                 self._selection_surface = None
                 return
-            y = self._font.size(self._label)[1]
+            y = self._font.size(self._title)[1]
 
             # Add ellipsis
             delta = self._ellipsis_size
@@ -429,8 +447,7 @@ class TextInput(Widget):
             x2 += delta
 
             # Create surface and fill
-            self._selection_surface = make_surface(x, y)
-            self._selection_surface.fill(self._selection_color)
+            self._selection_surface = make_surface(x, y, fill_color=self._selection_color)
             self._selection_position[0] = x1 + self._rect.x
             self._selection_position[1] = self._rect.y
 
@@ -481,36 +498,38 @@ class TextInput(Widget):
             current_rect = self._surface.get_rect()  # type: pygame.Rect
             menu_rect = menu.get_rect()
             posx2 = menu_rect.x + menu_rect.width
-            space_between_label = posx2 - self._label_size - self._rect.x
-            char = math.ceil(space_between_label * 1.0 / self._input_underline_size)  # floor does not work
+            space_between_title = posx2 - self._title_size - self._rect.x
+            char = math.ceil(space_between_title * 1.0 / self._input_underline_size)  # floor does not work
 
             # If char limit
             max_width_current = 0
             if self._maxchar != 0 or self._maxwidth != 0:
                 max_chars = max(self._maxchar, self._maxwidth_base)
                 basechar = 'O'
+                multif = 4  # Factor of ellipsis
                 if self._password:
                     basechar = self._password_char
-                max_size = self.font_render_string(basechar * max_chars)
+                    multif = 2
+                max_size = self._font_render_string(basechar * max_chars)
                 max_size = max_size.get_size()[0]
-                maxchar_char = math.ceil((max_size + 4 * self._ellipsis_size) / self._input_underline_size)
+                maxchar_char = math.ceil((max_size + multif * self._ellipsis_size) / self._input_underline_size)
                 char = min(char, maxchar_char)
                 max_width_current = current_rect.width
 
             underline_string = self._input_underline * int(char)
 
             # Render char
-            underline = self.font_render_string(underline_string, color)
+            underline = self._font_render_string(underline_string, color, use_background_color=False)
 
             # Create a new surface
-            new_width = max(self._label_size + underline.get_size()[0],
+            new_width = max(self._title_size + underline.get_size()[0],
                             max_width_current,
                             self._last_rendered_surface_underline_width)
             new_surface = make_surface(new_width + 1, current_rect.height + 3, alpha=True)
 
             # Blit current surface
             new_surface.blit(self._surface, (0, 0))
-            new_surface.blit(underline, (self._label_size - 1, 6))  # Position (x, y)
+            new_surface.blit(underline, (self._title_size - 1, 6))  # Position (x, y)
             self._last_rendered_surface_with_underline = new_surface
             self._last_rendered_surface_underline_width = new_width
         else:
@@ -541,10 +560,10 @@ class TextInput(Widget):
         # Calculate x position
         if self._maxwidth == 0:  # If no limit is provided
             cursor_x_pos = self._cursor_offset + \
-                           self._font.size(self._label + string[:self._cursor_position])[0]
+                           self._font.size(self._title + string[:self._cursor_position])[0]
         else:  # Calculate position depending on renderbox
             string = string[self._renderbox[0]:(self._renderbox[0] + self._renderbox[2])]
-            cursor_x_pos = self._cursor_offset + self._font.size(self._label + string)[0]
+            cursor_x_pos = self._cursor_offset + self._font.size(self._title + string)[0]
 
             # Add ellipsis
             delta = self._ellipsis_size
@@ -557,7 +576,7 @@ class TextInput(Widget):
             else:
                 delta *= 0
             cursor_x_pos += delta
-        if self._cursor_position > 0 or (self._label and self._cursor_position == 0):
+        if self._cursor_position > 0 or (self._title and self._cursor_position == 0):
             # Without this, the cursor is invisible when self._cursor_position > 0:
             cursor_x_pos -= self._cursor_surface.get_width()
 
@@ -830,7 +849,7 @@ class TextInput(Widget):
         cursor_pos = len(string)
         for i in range(len(string)):
             size_sum += string_size[i] / 2
-            if self._label_size + size_sum >= mousex:
+            if self._title_size + size_sum >= mousex:
                 cursor_pos = i
                 break
             size_sum += string_size[i] / 2
@@ -1032,18 +1051,25 @@ class TextInput(Widget):
         """
         return self._input_string[self._selection_box[0]:self._selection_box[1]]
 
-    def _update_input_string(self, new_string):
+    def _update_input_string(self, new_string, update_history=True):
         """
         Update input string with a new string, store changes into history.
 
         :param new_string: New string of text input
         :type new_string: str
+        :param update_history: Updates history
+        :type update_history: bool
         :return: None
         """
+        assert isinstance(new_string, str)
+        assert isinstance(update_history, bool)
+
         l_history = len(self._history)
 
         # If last edition is different than the new one -> updates the history
-        if ((l_history > 0 and self._history[l_history - 1] != new_string) or l_history == 0) and self._max_history > 0:
+        if update_history and \
+                ((l_history > 0 and self._history[
+                    l_history - 1] != new_string) or l_history == 0) and self._max_history > 0:
 
             # If index is not at last add the current status as new
             if self._history_index != l_history:
@@ -1115,7 +1141,7 @@ class TextInput(Widget):
         """
         if char in self._keychar_size.keys():
             return self._keychar_size[char]
-        self._keychar_size[char] = self.font_render_string(char).get_size()[0]
+        self._keychar_size[char] = self._font_render_string(char).get_size()[0]
         return self._keychar_size[char]
 
     def _paste(self):
@@ -1167,8 +1193,8 @@ class TextInput(Widget):
                 return False
 
         new_string = self._input_string[0:self._cursor_position] + \
-            text[0:text_end] + \
-            self._input_string[self._cursor_position:len(self._input_string)]
+                     text[0:text_end] + \
+                     self._input_string[self._cursor_position:len(self._input_string)]
 
         # If string is valid
         if self._check_input_type(new_string):
@@ -1244,40 +1270,44 @@ class TextInput(Widget):
 
         for i in range(removed):
             if left:
-                self._delete()
+                self._delete(update_history=i == removed - 1)
             else:
-                self._backspace()
+                self._backspace(update_history=i == removed - 1)
 
         # Destroy selection
         self._unselect_text()
 
-    def _backspace(self):
+    def _backspace(self, update_history=True):
         """
         Backspace event.
 
+        :param update_history: Updates history on deletion
+        :type update_history: bool
         :return: None
         """
         new_string = (
                 self._input_string[:max(self._cursor_position - 1, 0)]
                 + self._input_string[self._cursor_position:]
         )
-        self._update_input_string(new_string)
+        self._update_input_string(new_string, update_history=update_history)
         self._update_renderbox(left=-1, addition=True)
 
         # Subtract one from cursor_pos, but do not go below zero:
         self._cursor_position = max(self._cursor_position - 1, 0)
 
-    def _delete(self):
+    def _delete(self, update_history=True):
         """
         Delete event.
 
+        :param update_history: Updates history on deletion
+        :type update_history: bool
         :return: None
         """
         new_string = (
                 self._input_string[:self._cursor_position]
                 + self._input_string[self._cursor_position + 1:]
         )
-        self._update_input_string(new_string)
+        self._update_input_string(new_string, update_history=update_history)
         self._update_renderbox(right=-1, addition=True)
 
     def _select_all(self):
@@ -1362,6 +1392,7 @@ class TextInput(Widget):
     # noinspection PyMissingOrEmptyDocstring
     def update(self, events):
         updated = False
+        events = self._merge_events(events)  # Extend events with custom events
 
         for event in events:  # type: pygame.event.Event
 
@@ -1619,19 +1650,22 @@ class TextInput(Widget):
                 self._key_is_pressed = False
 
             elif self.mouse_enabled and event.type == pygame.MOUSEBUTTONUP:
-                self._absolute_origin = getattr(event, 'origin', self._absolute_origin)
-                self._selection_active = False
-                self._check_mouse_collide_input(event.pos)
-                self._cursor_ms_counter = 0
+                if self._rect.collidepoint(*event.pos) and \
+                        self.get_selected_time() > 1.5 * self._keyrepeat_mouse_interval_ms:
+                    self._absolute_origin = getattr(event, 'origin', self._absolute_origin)
+                    self._selection_active = False
+                    self._check_mouse_collide_input(event.pos)
+                    self._cursor_ms_counter = 0
 
             elif self.mouse_enabled and event.type == pygame.MOUSEBUTTONDOWN:
-                self._absolute_origin = getattr(event, 'origin', self._absolute_origin)
-                if self._selection_active:
-                    self._unselect_text()
-                self._cursor_ms_counter = 0
-                self._selection_active = True
-                self._selection_mouse_first_position = -1
-                self.active = True
+                if self.get_selected_time() > self._keyrepeat_mouse_interval_ms:
+                    self._absolute_origin = getattr(event, 'origin', self._absolute_origin)
+                    if self._selection_active:
+                        self._unselect_text()
+                    self._cursor_ms_counter = 0
+                    self._selection_active = True
+                    self._selection_mouse_first_position = -1
+                    self.active = True
 
         # Get time clock
         time_clock = self._clock.get_time()
@@ -1659,14 +1693,14 @@ class TextInput(Widget):
                 event_key, event_unicode = key, self._keyrepeat_counters[key][1]
                 try:
                     # noinspection PyArgumentList
-                    pygame.event.post(pygame.event.Event(pygame.KEYDOWN,
-                                                         key=event_key,
-                                                         unicode=event_unicode)
-                                      )
+                    self._add_event(
+                        pygame.event.Event(pygame.KEYDOWN,
+                                           key=event_key,
+                                           unicode=event_unicode)
+                    )
                 except pygame.error:  # If the keys are too fast pygame can raise a Sound Exception
                     pass
 
-        # Update self._cursor_visible
         self._cursor_ms_counter += time_clock
         if self._cursor_ms_counter >= self._cursor_switch_ms:
             self._cursor_ms_counter %= self._cursor_switch_ms
